@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2023 -- leonerd@leonerd.org.uk
 
-package XS::Parse::Keyword::FromPerl 0.02;
+package XS::Parse::Keyword::FromPerl 0.03;
 
 use v5.26; # XS code needs op_class() and the OPclass_* constants
 use warnings;
@@ -286,12 +286,37 @@ A package name, returned as a string in the I<sv> field.
 
 The C<_OPT> variant is optional.
 
+=head3 XPK_LEXVARNAME
+
+   XPK_LEXVARNAME($kind)
+
+A lexical variable name, returned as a string in the I<sv> field.
+
+The C<$kind> must be a bitmask of C<XPK_LEXVAR_SCALAR>, C<XPK_LEXVAR_ARRAY>,
+C<XPK_LEXVAR_HASH>; or C<XPK_LEXVAR_ANY> for convenience to set all three.
+
 =head3 XPK_VSTRING, XPK_VSTRING_OPT
 
 A version string, returned as a L<version> object instance in the I<sv> field.
 
 The C<_OPT> variant is optional.
 
+=head3 XPK_LEXVAR
+
+   XPK_LEXVAR($kind)
+
+A lexical variable that already exists in the pad, returned as a pad offset in
+the I<padix> field.
+
+C<$kind> is specified as in C<XPK_LEXVARNAME>.
+
+=head3 XPK_LEXVAR_MY
+
+   XPK_LEXVAR_MY($kind)
+
+A lexical variable, parsed as if it appeared in a C<my> expression. It will be
+added to the pad and returned as a pad offset in the I<padix> field.
+
 =head3 XPK_COMMA
 
 =head3 XPK_COLON
@@ -301,6 +326,115 @@ The C<_OPT> variant is optional.
 A literal comma, colon or equals sign. These do not appear in the arguments
 list.
 
+=head3 XPK_LITERAL
+
+   XPK_LISTEXPR($string)
+
+A literal string match. No value is returned.
+
+This should be avoided if at all possible, in favour of the character matches
+above, or C<XPK_KEYWORD>.
+
+=head3 XPK_KEYWORD
+
+   XPK_KEYWORD($string)
+
+A literal string match, which requires that the following text does not begin
+with an identifier character (thus avoiding prefix-match problems). No value
+is returned.
+
+=head3 XPK_SEQUENCE
+
+   XPK_SEQUENCE(@pieces)
+
+A sub-sequence. Normally this is not necessary, as most of the
+structure-forming functions already take a sequence of pieces. It is mostly
+useful as as an option to the C<XPK_CHOICE> function.
+
+Nothing extra is returned, beyond the values from the individual pieces.
+
+=head3 XPK_OPTIONAL
+
+   XPK_OPTIONAL(@pieces)
+
+An optional sequence of pieces that may or may not be present. Returns an
+integer value in the I<i> field of 0 if the sequence was not found, or 1
+followed by its values if the sequence was found.
+
+=head3 XPK_REPEATED
+
+   XPK_REPEATED(@pieces)
+
+A repeating sequence of pieces. Returns an integer value in the I<i> field
+indicating how many times the sequence repeated, followed by all the values
+returned by each.
+
+=head3 XPK_CHOICE
+
+   XPK_CHOICE(@pieces)
+
+The pieces of this function are not interpreted as a sequence, but instead
+as a list of possible choices. Returns an integer value in the I<i> field
+indicating which choice was found, followed by all the values returned by
+that sequence.
+
+The first possible choice is numbered C<0>. If no choice matched, it returns
+C<-1>. To cause an error instead, use L<XPK_FAILURE>.
+
+=head3 XPK_FAILURE
+
+   XPK_FAILURE($message)
+
+Attempting to parse this piece type will immediately cause a compile-time
+failure with the given message. This can be used as the final option in
+C<XPK_CHOICE> to ensure a valid match.
+
+=head3 XPK_PARENSCOPE
+
+   XPK_PARENSCOPE(@pieces)
+
+Expects to find a sequence of pieces, all surrounded by parentheses (round
+brackets, C<( ... )>).
+
+Nothing extra is returned, beyond the values from the individual contained
+pieces.
+
+=head3 XPK_ARGSCOPE
+
+   XPK_ARGSCOPE(@pieces)
+
+A scope similar to C<XPK_PARENSCOPE> except that the parentheses themselves
+are optional, similar to perl's parsing of calls to known functions.
+
+=head3 XPK_BRACKETSCOPE
+
+   XPK_BRACKETSCOPE(@pieces)
+
+Expects to find a sequence of pieces, all surrounded by square brackets
+(C<[ ... ]>).
+
+Nothing extra is returned, beyond the values from the individual contained
+pieces.
+
+=head3 XPK_BRACESCOPE
+
+   XPK_BRACESCOPE(@pieces)
+
+Expects to find a sequence of pieces, all surrounded by braces (C<{ ... }>).
+
+Nothing extra is returned, beyond the values from the individual contained
+pieces.
+
+=head3 XPK_CHEVRONSCOPE
+
+   XPK_CHEVRONSCOPE(@pieces)
+
+Expects to find a sequence of pieces, all surrounded by chevrons (angle
+brackets, C<< < ... > >>).
+
+Nothing extra is returned, beyond the values from the individual contained
+pieces.
+
 =cut
 
 # Simple pieces
@@ -314,6 +448,27 @@ foreach (qw(
    no strict 'refs';
    *$name = sub { bless [$name], "XS::Parse::Keyword::FromPerl::_Piece" };
 }
+# Single-SV parametric pieces
+foreach (qw(
+      LEXVARNAME LEXVAR LEXVAR_MY LITERAL KEYWORD FAILURE
+   )) {
+   my $name = "XPK_$_";
+   push @EXPORT_OK, $name;
+
+   no strict 'refs';
+   *$name = sub { bless [$name, $_[0]], "XS::Parse::Keyword::FromPerl::_Piece" };
+}
+# Structural multiple-value pieces
+foreach (qw(
+      SEQUENCE OPTIONAL REPEATED CHOICE
+      PARENSCOPE ARGSCOPE BRACKETSCOPE BRACESCOPE CHEVRONSCOPE
+   )) {
+   my $name = "XPK_$_";
+   push @EXPORT_OK, $name;
+
+   no strict 'refs';
+   *$name = sub { bless [$name, [@_]], "XS::Parse::Keyword::FromPerl::_Piece" };
+}
 
 =head2 Parser Arguments
 
@@ -369,8 +524,8 @@ Returns the line number of the source text on which the piece was started.
 
 =item *
 
-Non-trivial C<pieces> for parsing. Pieces that are parametric (e.g. LEXVAR),
-or pieces that are structural and consume other pieces (OPTIONAL, SEQ, etc..)
+Non-trivial C<pieces> for parsing. Pieces that are structural and consume
+other pieces (OPTIONAL, SEQ, etc..)
 
 =item *
 
